using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Xml;
using System.Runtime.Serialization;
using System.Xml.Serialization;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Channels;
using System.ServiceModel;
//
using RKiss.WSEventing;


namespace Subscriber
{
    class Program
    {
        static void Main(string[] args)
        {
            while (true)
            {
                try
                {

                    using (SubscriptionManagerClient manager = new SubscriptionManagerClient("SubscriptionManager"))
                    {                     
                        #region Operation Subscribe
                        Console.WriteLine("\nSubscribe - Press any key to continue"); Console.ReadKey();
                        /*******************************************************************\
                            Example of the Subscription for weather report
                        \*******************************************************************/

                        // Prepare request for subscribing subscription
                        SubscribeRequest subscribeRequest = new SubscribeRequest();
                        
                        // option: Application specific header
                        AddressHeader[] ah = new AddressHeader[1] { AddressHeader.CreateAddressHeader("MySubscription", "urn:MyNamespace", "1234567890") };

                        // option: organize subscriptions in the storage based on the topic
                        subscribeRequest.SubscriptionTopic = "weather:report.storm";
                        // or another option
                        //subscribeRequest.Delivery.Add(AddressHeader.CreateAddressHeader("EventTopic", "", "weather:report.storm"));
                        
                        // Delivery:
                        subscribeRequest.Delivery.DeliveryMode = WSEventing.PushDeliveryMode;
                        EndpointAddress notifyTo = new EndpointAddress(new Uri("net.tcp://localhost:33333/OnStormWarning"), ah);
                        subscribeRequest.Delivery.NotifyTo = notifyTo;
                    
                        // EndTo:
                        subscribeRequest.EndTo = new EndpointAddress(new Uri("net.tcp://localhost:44444/Admin"), ah);

                        // Filter:
                        XmlNamespaceManager nsmgr = new XmlNamespaceManager(new XmlDocument().NameTable);
                        nsmgr.AddNamespace("ow", "http://www.example.org/oceanwatch");
                        subscribeRequest.Filter = new XPathMessageFilter("//ow:WindReport/ow:Speed >= 65", nsmgr);

                        // lifetime of the subscription
                        subscribeRequest.Expires = new Expires(TimeSpan.FromSeconds(300));
                        //subscribeRequest.Subscribe.Expires = new Expires(DateTime.Now.Add(TimeSpan.FromSeconds(300)));
                        
                        // show me request
                        Util.ShowMe(subscribeRequest);

                        // action - Subscribe
                        SubscribeResponse subscribeResponse = manager.Subscribe(subscribeRequest);

                        // show me response
                        Util.ShowMe(subscribeResponse);

                        // another subscribing (using advanced API)
                        //Console.WriteLine("\nSubscribe again - Press any key to continue"); Console.ReadKey();
                        //SubscribeResponse subscribeResponse2 = manager.Subscribe("weather:report.storm", notifyTo, TimeSpan.FromMinutes(10));
                        #endregion

                        // my subscription identifier
                        Identifier id = subscribeResponse.SubscriptionManager.Identifier;

                        #region Operation - GetStatus
                        Console.WriteLine("\nGetStatus - Press any key to continue"); Console.ReadKey();
                        GetStatusRequest getstatusRequest = new GetStatusRequest(id);
                        GetStatusResponse getstatusResponse = manager.GetStatus(getstatusRequest);
                        Util.ShowMe(getstatusResponse);
                        #endregion

                        #region Operation - Renew
                        Console.WriteLine("\nRenew - Press 'Y' to continue or any key to skip it.");
                        if (Console.ReadKey().KeyChar == 'y')
                        {
                            RenewRequest renewRequest = new RenewRequest(id, new Renew(new Expires(TimeSpan.FromSeconds(120))));
                            RenewResponse renewResponse = manager.Renew(renewRequest);
                            Util.ShowMe(renewResponse);
                        }
                        #endregion

                        #region Operation - GetStatus
                        Console.WriteLine("\nGetStatus - Press any key to continue"); Console.ReadKey();
                        GetStatusRequest getstatusRequest2 = new GetStatusRequest(id);
                        GetStatusResponse getstatusResponse2 = manager.GetStatus(getstatusRequest2);
                        Util.ShowMe(getstatusResponse2);
                        #endregion

                        #region Operation Unsubscribe
                        Console.WriteLine("\nUnsubscribe - Press 'Y' to continue or any key to skip it.");
                        if (Console.ReadKey().KeyChar == 'y')
                        {
                            UnsubscribeRequest unsubscribeRequest = new UnsubscribeRequest(id);
                            UnsubscribeResponse unsubscribeResponse = manager.Unsubscribe(unsubscribeRequest);
                            Util.ShowMe(unsubscribeResponse);
                        }
                        #endregion

                        Console.WriteLine("\nDone, Press any key to test again"); Console.ReadKey();
                    }

                }
                catch (FaultException<SupportedDialect> fe)
                {
                    Console.WriteLine(fe);
                }
                catch (FaultException<SupportedDeliveryMode> fe)
                {
                    Console.WriteLine(fe);
                }
                catch (FaultException fe)
                {
                    Console.WriteLine(fe);
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex);
                }
            }

            //Console.ReadLine();
        }
    }

    public sealed class Util
    {
        private Util() { }
        public static void ShowMe(object resource)
        {
            Console.WriteLine();
            if (resource != null)
            {
                XmlTextWriter xtw = new XmlTextWriter(Console.Out);
                xtw.Formatting = Formatting.Indented;
                XmlSerializer xs = new XmlSerializer(resource.GetType());
                xs.Serialize(xtw, resource);
            }
            Console.WriteLine();
        }
    }
}


