using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace EventSink
{

    #region Weather
    [ServiceContract]
    [XmlSerializerFormat]
    public interface IWeather
    {
        [OperationContract(Action = "http://www.example.org/oceanwatch/WindReport", IsOneWay = true)]
        void WindReport(Message request);
    }
    public class Weather : IWeather
    {
        public void WindReport(Message request)
        {
            Console.WriteLine(request);
        }
    }
    #endregion

    #region Admin
    [ServiceContract]
    [XmlSerializerFormat]
    public interface IAdmin
    {
        [OperationContract(Action = "http://schemas.xmlsoap.org/ws/2004/08/eventing/SubscriptionEnd", IsOneWay = true)]
        void SubscriptionEnd(Message request);
    }
    public class Admin : IAdmin
    {
        public void SubscriptionEnd(Message request)
        {
            Console.ForegroundColor = ConsoleColor.Magenta;
            Console.WriteLine(request);
            Console.ResetColor();
        }
    }
    #endregion


    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                using (ServiceHost host = new ServiceHost(typeof(Weather)))
                using (ServiceHost host2 = new ServiceHost(typeof(Admin)))
                {
                    host.Open();
                    host2.Open();

                    Console.WriteLine("Press any key to stop server(s)...");
                    Console.ReadLine();

                    host.Close();
                    host2.Close();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                Console.ReadLine();
            }
        }
    }
}
