
#region References
using System;
using System.Globalization;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Xml;
using System.Runtime.Serialization;
using System.Xml.Serialization;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Channels;
using System.ServiceModel;
#endregion

namespace EventSource
{
    #region Example of an Event Source
    /*******************************************************************\
        Example of the Event Source - ocean watch
    \*******************************************************************/

    [ServiceContract]
    [XmlSerializerFormat]
    public interface IWeather
    {
        [OperationContract(Action = "http://www.example.org/oceanwatch/WindReport", IsOneWay = true)]
        void WindReport(WindReportRequest request);
    }

    [MessageContract(IsWrapped=false)]
    public class WindReportRequest
    {
        [MessageBodyMember]
        public WindReport WindReport;
        public WindReportRequest() { }
        public WindReportRequest(WindReport report) { WindReport = report; }
    }

    [XmlSchemaProvider(null, IsAny = true)]
    public class WindReport : IXmlSerializable
    {
        string _namespace = "http://www.example.org/oceanwatch";
        DateTime _datetime;
        string _location;
        int _speed;
        string _comments;
        
        public DateTime DateTime { get { return _datetime; } set { _datetime = value; } }
        public string Location { get { return _location; } set { _location = value; } }
        public int Speed { get { return _speed; } set { _speed = value; } }
        public string Comments { get { return _comments; } set { _comments = value; } }
        public WindReport() { }

        #region IXmlSerializable Members
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            throw new Exception("The method or operation is not implemented.");   
        }
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteStartElement("ow", "WindReport", _namespace);
            writer.WriteElementString("DateTime", _namespace, XmlConvert.ToString(DateTime, XmlDateTimeSerializationMode.Utc));
            writer.WriteElementString("Location", _namespace, Location);
            writer.WriteElementString("Speed", _namespace, XmlConvert.ToString(Speed));
            writer.WriteElementString("Comments", _namespace, Comments);
            writer.WriteEndElement();
        }
        #endregion
    }
    #endregion


    #region CancelSubscription Contract
    [ServiceContract]
    public interface INotificationManager
    {
        [OperationContract(Action = "CancelSubscription", IsOneWay = true)]
        void CancelSubscription(Uri code, string reason, string culturename);
    }
    #endregion


    class Program
    {
        static void Main(string[] args)
        {
            
            try
            {
                #region Event Simulation 
                while (true)
                {
                    int speed = 65;
                    Console.WriteLine("\nSend event (WindReport). Type value of a speed or 'c' to cancel subscription(s)");
                    string input = Console.ReadLine();
                    if (input == "c")
                    {
                        using (ChannelFactory<INotificationManager> channelNotification = new ChannelFactory<INotificationManager>("Notify"))
                        {
                            channelNotification.Open();
                            INotificationManager notify = channelNotification.CreateChannel();
                            notify.CancelSubscription(new Uri("http://schemas.xmlsoap.org/ws/2004/08/eventing/SourceCanceling"), "Testing", CultureInfo.CurrentCulture.Name);
                            channelNotification.Close();
                        }   
                    }
                    else
                    {
                        int.TryParse(input, out speed);
                        using (ChannelFactory<IWeather> channelNotification = new ChannelFactory<IWeather>("Notify"))
                        {
                            channelNotification.Open();
                            IWeather manager = channelNotification.CreateChannel();
                            WindReportRequest request = new WindReportRequest(new WindReport());
                            request.WindReport.DateTime = DateTime.Now;
                            request.WindReport.Location = "Huntington Beach, CA";
                            request.WindReport.Speed = speed == 0 ? 65 : speed;
                            request.WindReport.Comments = "WINDS 55 WITH GUSTS TO 65, ROOF TORN OFF";
                            manager.WindReport(request);
                            channelNotification.Close();
                        }
                    }
                }
                #endregion
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                Console.ReadLine();
            }
        }
    }
}


