//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)        
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//*****************************************************************************
//  
#region References
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ServiceModel;
using System.Runtime.Serialization;
using System.Xml;
using System.Xml.Serialization;
#endregion

namespace RKiss.WSEventing.Adapters
{

    [Serializable]
    public sealed class ResourceDeletedEventArgs : EventArgs
    {
        private object _resource;
        private ResourceDescriptor _rd;
        public object Resource
        {
            get { return _resource; }
            set { _resource = value; }
        }
        public ResourceDescriptor ResourceDescriptor
        {
            get { return _rd; }
            set { _rd = value; }
        }

        public ResourceDeletedEventArgs(ResourceDescriptor rd, object resource)
        {
            ResourceDescriptor = rd;
            Resource = resource;
        }
    }

    [Serializable]
    [XmlSerializerFormat]   
    [DataContract(Name = "ResourceDescriptor", Namespace = "http://www.rkiss.net/schemas/sb/2005/06/servicebus")]
    public class ResourceDescriptor
    {
        #region DataMembers
        UniqueId _id;
        string _name;
        string _key;
        string _topic;
        DateTime _expires;
        DateTime _created;
        DateTime _updated;
        #endregion

        #region Properties
        [DataMember(Order = 0)]
        public UniqueId Id
        {
            get { return _id; }
            set { _id = value; }
        }
        [DataMember(Order = 1)]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }
        [DataMember(Order = 2)]
        public string Key
        {
            get { return _key; }
            set { _key = value; }
        }
        [DataMember(Order = 3)]
        public string Topic
        {
            get { return _topic; }
            set { _topic = value; }
        }
        [DataMember(Order = 4)]
        public DateTime Expires
        {
            get { return _expires; }
            set { _expires = value; }
        }
        [DataMember(Order = 5)]
        public DateTime Created
        {
            get { return _created; }
            set { _created = value; }
        }
        [DataMember(Order = 6)]
        public DateTime Updated
        {
            get { return _updated; }
            set { _updated = value; }
        }
        #endregion

        #region Constructors
        public ResourceDescriptor() { }
        public ResourceDescriptor(UniqueId id) { Id = id; }
        public ResourceDescriptor(UniqueId id, DateTime expires) { Id = id; Expires = expires; }
        public ResourceDescriptor(ResourceDescriptor rd)
        {
            if (rd != null)
            {
                // copy instance
                if(rd.Id != null)
                    Id = new UniqueId(rd.Id.ToString());
                if(rd.Name != null)
                    Name = string.Copy(rd.Name);
                if(rd.Key != null)
                    Key = string.Copy(rd.Key);
                if(rd.Topic != null)
                    Topic = string.Copy(rd.Topic);
                Expires = rd.Expires;
                Created = rd.Created;
                Updated = rd.Updated;
            }
        }
        #endregion
    }

    public interface IStorageAdapter
    {
        // basic
        ResourceDescriptor Create<T>(ResourceDescriptor rd, T resource) where T : class;
        T Get<T>(ResourceDescriptor rd) where T : class;
        void Put<T>(ResourceDescriptor rd, T resource) where T : class;
        T Delete<T>(ResourceDescriptor rd) where T : class;


        // advanced methods
        void CreateOrUpdate<T>(ResourceDescriptor rd, T resource) where T : class;
        IList<T> Get<T>(string topic) where T : class;
        IList<T> Delete<T>(string topic) where T : class;
        T Delete<T>(UniqueId id);
        ResourceDescriptor GetResourceDescriptor(ResourceDescriptor rd);
        ResourceDescriptor[] GetResourceDescriptors(ResourceDescriptor rd);
        void Expires(ResourceDescriptor rd);   
    }
}
