//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)    
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
#region References
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Channels;
#endregion

namespace RKiss.WSEventing
{
    #region State objects
    public class FireState
    {
        HybridDictionary _properties;
        Message _message;
        Subscription _subscription;
        public HybridDictionary Properties { get { return _properties; } }
        public Message Message { get { return _message; } }
        public Subscription Subscription { get { return _subscription; } }
        public FireState(Subscription subscription, Message message, HybridDictionary properties)
        {
            _properties = properties;
            _subscription = subscription;
            _message = message;
        }
    }

    public class CancelState
    {
        HybridDictionary _properties;
        SubscriptionEnd _subscriptionEnd;
        EndpointAddress _endTo;
        public HybridDictionary Properties { get { return _properties; } }
        public SubscriptionEnd SubscriptionEnd { get { return _subscriptionEnd; } }
        public EndpointAddress EndTo { get { return _endTo; } }
        public CancelState(EndpointAddress endTo, SubscriptionEnd subscriptionEnd, HybridDictionary properties)
        {
            _properties = properties;
            _endTo = endTo;
            _subscriptionEnd = subscriptionEnd;
        }
    }
    #endregion

    public sealed class Convert
    {
        private Convert() { }
        public static T XmlToObject<T>(object resource)
        {
            if (resource is XmlElement && typeof(T) != typeof(object))
            {
                XmlElement element = resource as XmlElement;
                XmlSerializer xs = new XmlSerializer(typeof(T));
                using (MemoryStream ms = new MemoryStream())
                {
                    XmlTextWriter xtw = new XmlTextWriter(ms, Encoding.UTF8);
                    element.WriteTo(xtw);
                    xtw.Flush();
                    ms.Position = 0;
                    return (T)xs.Deserialize(ms);
                }
            }
            else
            {
                return (T)resource;
            }
        }
        public static XmlElement ObjectToXml(object resource)
        {
            if (resource != null)
            {
                if (resource is XmlElement)
                {
                    return resource as XmlElement;
                }
                else if (resource is XmlNode)
                {
                    return resource as XmlElement;
                }
                else
                {
                    XmlDocument doc = new XmlDocument();
                    using (MemoryStream ms = new MemoryStream())
                    {
                        XmlWriter xtw = XmlTextWriter.Create(ms, null);
                        XmlSerializer xs = new XmlSerializer(resource.GetType());
                        xs.Serialize(xtw, resource);
                        ms.Position = 0;
                        doc.Load(ms);
                    }
                    return doc.DocumentElement;
                }
            }
            return null;
        }
        /// <summary>
        /// Serialize object into the xml wrapper
        /// </summary>
        /// <param name="resource"></param>
        /// <returns></returns>
        public static XmlElement ObjectToXmlAnyElement(object resource)
        {
            if (resource != null)
            {
                if (resource is XmlElement)
                {
                    return resource as XmlElement;
                }
                else if (resource is XmlNode)
                {
                    return resource as XmlElement;
                }
                else
                {
                    XmlDocument doc = new XmlDocument();
                    using (MemoryStream ms = new MemoryStream())
                    {
                        XmlWriter xtw = XmlTextWriter.Create(ms, null);
                        xtw.WriteStartElement("root");
                        XmlSerializer xs = new XmlSerializer(resource.GetType());
                        xs.Serialize(xtw, resource);
                        xtw.WriteEndElement();
                        xtw.Flush();
                        ms.Position = 0;
                        doc.Load(ms);
                    }
                    return doc.DocumentElement;
                }
            }
            return null;
        }

        public static SubscriptionEndCode ToSubscriptionEndCode(Uri uricode)
        {
            if (uricode.AbsoluteUri == WSEventing.DeliveryFailure)
                return SubscriptionEndCode.DeliveryFailure;
            else if (uricode.AbsoluteUri == WSEventing.SourceCanceling)
                return SubscriptionEndCode.SourceCanceling;
            else if (uricode.AbsoluteUri == WSEventing.SourceShuttingDown)
                return SubscriptionEndCode.SourceShuttingDown;
            else
                throw new FaultException("Not recognize code");
        }
    }

    public class HeaderCollection : List<AddressHeader>
    {
        public AddressHeader Find(string name, string ns, bool throwex)
        {
            if (name == null)
                throw new ArgumentNullException("name");
            if (ns == null)
                throw new ArgumentNullException("ns");

            for (int ii = 0; ii < base.Count; ii++)
            {
                AddressHeader header = base[ii];
                if (header.Name == name && header.Namespace == ns)
                    return header;
            }
            if(throwex)
                throw new Exception(string.Format("The header name={0}, ns={1} doesn't exist", name, ns));
            return null;
        }
        public T GetValue<T>(string name, string ns, bool throwex)
        {
            AddressHeader ah = Find(name, ns, throwex);
            return ah == null ? default(T) : ah.GetValue<T>();
        }
    }

    public sealed class Util
    {
        private Util() { }
        public static void ShowMe(object resource)
        {
            Console.WriteLine();
            if (resource != null)
            {
                XmlTextWriter xtw = new XmlTextWriter(Console.Out);
                xtw.Formatting = Formatting.Indented;
                XmlSerializer xs = new XmlSerializer(resource.GetType());
                xs.Serialize(xtw, resource);
            }
            Console.WriteLine();
        }
    }
}