//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)       
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
#region References
using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.Runtime.Serialization;
#endregion

namespace RKiss.WSEventing
{
    [XmlSchemaProvider(null, IsAny = true)]
    public class Delivery : HeaderCollection, IXmlSerializable
    {
        #region DataMembers
        string _mode;
        EndpointAddress _notifyTo;
        #endregion

        #region Properties
        public string DeliveryMode
        {
            get { return _mode; }
            set { _mode = value; }
        }
        public EndpointAddress NotifyTo
        {
            get { return _notifyTo; }
            set { _notifyTo = value; }
        }
        #endregion

        #region Constructors
        public Delivery()
            : this(WSEventing.PushDeliveryMode, null)
        {
        }
        public Delivery(string mode, EndpointAddress notifyTo)
        {
            DeliveryMode = mode;
            NotifyTo = notifyTo;
        }
        public Delivery(XmlReader reader)
        {
            this.ReadXml(reader);
        }
        #endregion

        #region IXmlSerializable Members
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            if (reader.IsStartElement(WSEventing.ElementNames.Delivery, WSEventing.NamespaceUri) == false)
                throw new Exception(string.Format("Invalid Element, it must be {0}", WSEventing.ElementNames.Delivery));

            string mode = reader.GetAttribute(WSEventing.ElementNames.Mode, WSEventing.NamespaceUri);
            DeliveryMode = mode == "" ? WSEventing.PushDeliveryMode : mode;

            reader.ReadToDescendant(WSEventing.ElementNames.NotifyTo, WSEventing.NamespaceUri);
            NotifyTo = EndpointAddress.ReadFrom(AddressingVersion.WSAddressing10, reader);
            if (NotifyTo == null)
                throw new Exception(string.Format("Missing element {0}", WSEventing.ElementNames.NotifyTo));

            // option: additional headers
            while (reader.NodeType != XmlNodeType.EndElement)
            {
                AddressHeader ah = AddressHeader.CreateAddressHeader(reader.Name, reader.NamespaceURI, reader.ReadElementContentAsObject());
                base.Add(ah);
                reader.MoveToContent();
            }
            reader.ReadEndElement();
        }
        public void WriteXml(XmlWriter writer)
        {
            if (NotifyTo == null)
                throw new NullReferenceException("NotifyTo");

            writer.WriteStartElement(WSEventing.NamespacePrefix, WSEventing.ElementNames.Delivery, WSEventing.NamespaceUri);
            if (DeliveryMode != WSEventing.PushDeliveryMode)
            {
                writer.WriteAttributeString(WSEventing.NamespacePrefix, WSEventing.ElementNames.Mode, WSEventing.NamespaceUri, DeliveryMode);
            }
            NotifyTo.WriteTo(AddressingVersion.WSAddressing10, writer, WSEventing.ElementNames.NotifyTo, WSEventing.NamespaceUri);

            // option: additional headers
            for (int ii = 0; ii < base.Count; ii++)
            {
                base[ii].WriteAddressHeader(writer);
            }
            writer.WriteEndElement();
        }
        #endregion
    }
}




