//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)       
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
#region References
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.ServiceModel;
using System.Runtime.Serialization;
#endregion

namespace RKiss.WSEventing
{
    [XmlSchemaProvider(null, IsAny = true)]
    public class Expires : IXmlSerializable
    {
        #region DataMembers
        private string _expires = null;
        #endregion

        #region Properties
        public DateTime Value
        {
            get
            {
                if (_expires.StartsWith("P"))
                {
                    DateTime dt = DateTime.Now.Add(XmlConvert.ToTimeSpan(_expires)).ToUniversalTime();
                    _expires = XmlConvert.ToString(dt, XmlDateTimeSerializationMode.Utc);
                    return dt;
                }
                return XmlConvert.ToDateTime(_expires, XmlDateTimeSerializationMode.Utc);
            }
            set
            {
                _expires = XmlConvert.ToString(value, XmlDateTimeSerializationMode.Utc);
            }
        }
        #endregion

        #region Constructors
        public Expires(DateTime dt)
        {
            _expires = XmlConvert.ToString(dt, XmlDateTimeSerializationMode.Utc);
        }
        public Expires(TimeSpan duration)
        {
            _expires = XmlConvert.ToString(duration);
        }
        public Expires(XmlReader reader)
        {
            this.ReadXml(reader);
        }
        #endregion

        #region IXmlSerializable Members
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            if (reader.IsStartElement(WSEventing.ElementNames.Expires, WSEventing.NamespaceUri))
            {
                _expires = reader.ReadElementString();
            }
        }
        public void WriteXml(XmlWriter writer)
        {
            if (_expires != null)
            {
                writer.WriteElementString(WSEventing.NamespacePrefix, WSEventing.ElementNames.Expires, WSEventing.NamespaceUri, _expires);
            }
        }
        #endregion

        #region Validations
        public bool IsValidTime(bool throwex)
        {
            if (!_expires.StartsWith("P") && DateTime.Parse(_expires) < DateTime.Now)
            {
                if (throwex)
                {
                    throw new FaultException(WSEventing.InvalidExpirationType.Reason,
                        FaultCode.CreateSenderFaultCode(WSEventing.InvalidExpirationType.SubCode, WSEventing.NamespaceUri));
                }
                return false;
            }
            return true;
        }
        public bool IsDurationTime(bool throwex)
        {
            if (_expires.StartsWith("P"))
                return true;
            if (throwex)
                throw new FaultException(WSEventing.UnsupportedExpirationType.Reason,
                    FaultCode.CreateSenderFaultCode(WSEventing.UnsupportedExpirationType.SubCode, WSEventing.NamespaceUri));
            return false;
        }
        #endregion
    }
}
