//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)       
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
#region References
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.ServiceModel;
using System.Runtime.Serialization;
#endregion

namespace RKiss.WSEventing
{
    [XmlSchemaProvider(null, IsAny = true)]
    public class SupportedDeliveryMode : IXmlSerializable
    {
        private List<string> _modes;
        public List<string> Modes 
        {
            get { return _modes; }
            set { _modes = value; }
        }

        public SupportedDeliveryMode()
        {
            Modes = new List<string>();
        }
        public SupportedDeliveryMode(List<string> modes)
        {
            Modes = modes;
        }
      
        public bool IsSupported(string mode, bool throwex)
        {
            if (Modes == null || Modes.Count == 0 || mode == null)
                return true;

            if (Modes.Contains(mode))
                return true;

            if (throwex)
                throw new FaultException<SupportedDeliveryMode>(this, WSEventing.DeliveryModeRequestedUnvailable.Reason,
                   FaultCode.CreateSenderFaultCode(WSEventing.DeliveryModeRequestedUnvailable.SubCode, WSEventing.NamespaceUri));

            return false;
        }

        #region IXmlSerializable Members
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            reader.ReadStartElement(WSEventing.ElementNames.SupportedDeliveryMode, WSEventing.NamespaceUri);
            while (reader.NodeType != XmlNodeType.EndElement)
            {
                Modes.Add(reader.ReadContentAsString());
                reader.MoveToContent();
            }
            reader.ReadEndElement();
        }
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteStartElement(WSEventing.NamespacePrefix, WSEventing.ElementNames.SupportedDeliveryMode, WSEventing.NamespaceUri);
            if (Modes != null && Modes.Count > 0)
            {
                foreach (string mode in Modes)
                {
                    writer.WriteElementString(WSEventing.ElementNames.Mode, mode);
                }
            }
            writer.WriteEndElement();
        }
        #endregion
    }

    [XmlSchemaProvider(null, IsAny = true)]
    public class SupportedDialect : IXmlSerializable
    {
        private List<string> _dialects;
        public List<string> Dialects
        {
            get { return _dialects; }
            set { _dialects = value; }
        }

        public SupportedDialect()
        {
            Dialects = new List<string>();
        }
        public SupportedDialect(List<string> dialects)
        {
            Dialects = dialects;
        }
        public SupportedDialect(params string[] dialects) : this()
        {
            foreach (string dialect in dialects)
            {
                Dialects.Add(dialect);
            }
        }

        public bool IsSupported(string dialect, bool throwex)
        {
            if (dialect == null || dialect == "")
                return true;

            if (Dialects.Count == 0)
            {
                if (throwex)
                    throw new FaultException<SupportedDialect>(this, WSEventing.FilteringNotSupported.Reason,
                      FaultCode.CreateSenderFaultCode(WSEventing.FilteringNotSupported.SubCode, WSEventing.NamespaceUri));
                return false;
            }

            if (Dialects.Contains(dialect))
                return true;

            if (throwex)
                throw new FaultException<SupportedDialect>(this, WSEventing.FilteringRequestedUnavailable.Reason,
                   FaultCode.CreateSenderFaultCode(WSEventing.FilteringRequestedUnavailable.SubCode, WSEventing.NamespaceUri));

            return false;
        }

        #region IXmlSerializable Members
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            reader.ReadStartElement(WSEventing.ElementNames.SupportedDialect, WSEventing.NamespaceUri);
            while (reader.NodeType != XmlNodeType.EndElement)
            {
                Dialects.Add(reader.ReadContentAsString());
                reader.MoveToContent();
            }
            reader.ReadEndElement();
        }
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteStartElement(WSEventing.NamespacePrefix, WSEventing.ElementNames.SupportedDialect, WSEventing.NamespaceUri);
            if (Dialects != null && Dialects.Count > 0)
            {
                foreach (string dialect in Dialects)
                {
                    writer.WriteElementString(WSEventing.ElementNames.Dialect, dialect);
                }
            }
            writer.WriteEndElement();
        }
        #endregion
    }
}
