//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)        
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
#region References
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Runtime.Serialization;
#endregion

namespace RKiss.WSEventing
{
    #region Identifier
    [XmlSchemaProvider(null, IsAny = true)]
    public class Identifier : IXmlSerializable
    {
        #region DataMembers
        private UniqueId _identifier;
        #endregion

        #region Properties
        public UniqueId Value
        {
            get { return _identifier; }
            set { _identifier = value; }
        }
        public AddressHeader AddressHeader
        {
            get { return AddressHeader.CreateAddressHeader(WSEventing.ElementNames.Identifier, WSEventing.NamespaceUri, Value.ToString()); }
        }
        #endregion

        #region Constructors
        public Identifier()
        {
            Value = new UniqueId(string.Concat("uuid:", Guid.NewGuid()));
        }
        public Identifier(UniqueId id)
        {
            Value = id;
        }
        public Identifier(string id)
        {
            Value = new UniqueId(id);
        }
        public Identifier(Identifier id)
        {
            Value = id.Value;
        }
        public Identifier(XmlReader reader)
        {
            this.ReadXml(reader);
        }
        public Identifier(EndpointAddress epa)
        {
            AddressHeader header = epa.Headers.FindHeader(WSEventing.ElementNames.Identifier, WSEventing.NamespaceUri);
            this.ReadXml(header.GetAddressHeaderReader()); 
        }
        #endregion

        #region IXmlSerializable Members
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            Value = new UniqueId(reader.ReadElementContentAsString(WSEventing.ElementNames.Identifier, WSEventing.NamespaceUri));
        }
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteElementString(WSEventing.NamespacePrefix, WSEventing.ElementNames.Identifier, WSEventing.NamespaceUri, Value.ToString());
        }
        #endregion
    }
    #endregion
}
