//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)        
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
#region References
using System;
using System.Configuration;
using System.Collections.Specialized;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Xml.Serialization;
using System.Xml;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Configuration;
using System.Reflection;
#endregion

namespace RKiss.WSEventing
{
    #region PublisherMessageInspector
    public class PublisherMessageInspector : IDispatchMessageInspector
    {
        #region Properties
        Publishers _publishers;
        public Publishers Publishers
        {
            get { return _publishers; }
            set { _publishers = value; }
        }
        #endregion

        #region IStubMessageInspector
        public object AfterReceiveRequest(ref Message request, IClientChannel channel, InstanceContext instanceContext)
        {
            // store reference to the publishers into the context
            OperationContext.Current.IncomingMessageProperties["EventingPublishers"] = Publishers;
            return request;
        }
        public void BeforeSendReply(ref Message reply, object correlationState)
        {
            // nothing todo
        }
        #endregion
    }
    #endregion

    #region Publisher
    public class Publisher : Dictionary<string, string>, IXmlSerializable
    {
        #region Properties
        private bool enable;
        private string mode;
        private string name;
        private Type type;
        public bool Enable
        {
            get { return enable; }
            set { enable = value; }
        }
        public string Name
        {
            get { return name; }
            set { name = value; }
        }
        public string Mode
        {
            get { return mode; }
            set { mode = value; }
        }
        public Type Type
        {
            get { return type; }
            set { type = value; }
        } 
        #endregion

        #region Constructor
        public Publisher(XmlReader reader)
        {
            this.ReadXml(reader);
        } 
        #endregion

        #region IXmlSerializable Members
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            throw new Exception("The method or operation is not implemented.");
        }
        public void ReadXml(XmlReader reader)
        {
            if (reader.IsStartElement("publisher"))
            {
                // option: enable (default is true)
                string enableAttribute = reader.GetAttribute("enable");
                if (enable = System.Convert.ToBoolean(enableAttribute == null ? "true" : enableAttribute))
                {
                    // option: name
                    this.Name = reader.GetAttribute("name");

                    // mandatory: mode
                    this.Mode = reader.GetAttribute("mode");
                    if(this.Mode == null)
                        throw new Exception(string.Format("Missing attribute mode at element {0}", reader.Name));
                   
                    // option/mandatory: type
                    string type = reader.GetAttribute("type");
                    if (type != "")
                    {
                        this.Type = Type.GetType(type);
                        if (this.Type == null)
                            throw new Exception(string.Format("Can't create a type at element {0}", reader.Name));
                    }

                    // option: any attributes
                    if (reader.HasAttributes)
                    {
                        reader.MoveToFirstAttribute();
                        do
                        {
                            base.Add(reader.Name, reader.Value);
                        }
                        while (reader.MoveToNextAttribute());
                    }
                }
                reader.ReadElementString();
            }
            else
            {
                throw new Exception(string.Format("Failed at element {0}, expected element is {1}", reader.Name, "Filter"));
            }
        }
        public void WriteXml(XmlWriter writer)
        {
            throw new Exception("The method or operation is not implemented.");
        }
        #endregion
    }
    #endregion

    #region Publishers
    public class Publishers : Dictionary<string, Publisher>, IXmlSerializable
    {     
        public Publishers(XmlReader reader)
        {
            this.ReadXml(reader);
        }

        #region IXmlSerializable Member
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            while (reader.NodeType != XmlNodeType.EndElement)
            {
                if (reader.IsStartElement("publisher"))
                {
                    Publisher publisher = new Publisher(reader);
                    if (publisher.Enable)
                    {
                        base.Add(publisher.Mode, publisher);
                    }
                }
                else
                {
                    throw new Exception(string.Format("Failed at element {0}, expected elements are {1}", reader.Name, "publisher"));
                }
                reader.MoveToContent();
            }
            reader.ReadEndElement();    
        }
        public void WriteXml(XmlWriter writer)
        {
            throw new Exception("The method or operation is not implemented.");
        }
        #endregion
    }
    #endregion

    #region PublishersAttribute
    /// <summary>
    /// Plumbing class 
    /// </summary>
    public class PublishersAttribute : IEndpointBehavior, IServiceBehavior
    {
        #region Properties
        bool _enable = true;
        public bool Enable
        {
            get { return _enable; }
            set { _enable = value; }
        }

        Publishers _publishers;
        public Publishers Publishers
        {
            get { return _publishers; }
            set { _publishers = value; }
        }
        #endregion

        #region Constructors
        public PublishersAttribute()
        {
        }
        public PublishersAttribute(Publishers publishers)
        {
            Publishers = publishers;
        }
        #endregion

        #region IEndpointBehavior Members
        public void AddBindingParameters(ServiceEndpoint serviceEndpoint, BindingParameterCollection bindingParameters){}
        public void ApplyClientBehavior(ServiceEndpoint serviceEndpoint, ClientRuntime behavior) { }
        public void Validate(ServiceEndpoint serviceEndpoint) { }
        public void ApplyDispatchBehavior(ServiceEndpoint serviceEndpoint, EndpointDispatcher endpointDispatcher)
        {
            if (Enable)
            {
                PublisherMessageInspector inspector = new PublisherMessageInspector();
                inspector.Publishers = this.Publishers;
                endpointDispatcher.DispatchRuntime.MessageInspectors.Add(inspector);
            }
        }      
        #endregion

        #region IServiceBehavior Members
        public void AddBindingParameters(ServiceDescription description, ServiceHostBase serviceHostBase, Collection<ServiceEndpoint> endpoints, BindingParameterCollection parameters){}
        public void Validate(ServiceDescription description, ServiceHostBase serviceHostBase){}
        public void ApplyDispatchBehavior(ServiceDescription description, ServiceHostBase serviceHostBase)
        {
            foreach (ChannelDispatcher cd in serviceHostBase.ChannelDispatchers)
            {
                foreach (EndpointDispatcher epdispatch in cd.Endpoints)
                {
                    ApplyDispatchBehavior(null, epdispatch);
                }
            }
        }    
        #endregion
    }
    #endregion

    #region PublishersBehaviorElement
    /// <summary>
    /// Configuration class
    /// </summary>
    public class PublishersBehaviorElement : BehaviorExtensionElement
    {
        #region Properties
        Publishers _publishers;
        public Publishers Publishers
        {
            get { return _publishers; }
            set { _publishers = value; }
        }
        #endregion

        #region Constructor(s)
        public PublishersBehaviorElement()
        {
        }
        #endregion

        #region BehaviorExtensionElement
        protected override object CreateBehavior()
        {
            return new PublishersAttribute(this.Publishers);
        }
        public override void CopyFrom(ServiceModelExtensionElement from)
        {
            base.CopyFrom(from);
            PublishersBehaviorElement section = (PublishersBehaviorElement)from;
            this.Publishers = section.Publishers;
        }
        public override Type BehaviorType
        {
            get { return typeof(PublishersAttribute); }
        }

        /// <summary>
        /// Deserialize all childern elements - publisher
        /// </summary>
        /// <param name="elementName"></param>
        /// <param name="reader"></param>
        /// <returns>always true</returns>
        protected override bool OnDeserializeUnrecognizedElement(string elementName, XmlReader reader)
        {
            this.Publishers = new Publishers(reader);
            return true;
        }
        #endregion    
    }
    #endregion
}

