//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)      
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
#region References
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.Runtime.Serialization;
#endregion


namespace RKiss.WSEventing
{
    #region Subscribe
    [MessageContract]
    [XmlSchemaProvider(null, IsAny = true)]
    public class Subscribe : IXmlSerializable
    {
        #region DataMembers
        EndpointAddress _endTo;
        Delivery _delivery;
        Expires _expires;
        XPathMessageFilter _filter;
        string _filterDialect;
        #endregion

        #region Properties
        public EndpointAddress EndTo
        {
            get { return _endTo; }
            set { _endTo = value; }
        }
        public Delivery Delivery
        {
            get { return _delivery; }
            set { _delivery = value; }
        }
        public Expires Expires
        {
            get { return _expires; }
            set { _expires = value; }
        }
        public XPathMessageFilter Filter
        {
            get { return _filter; }
            set { _filter = value; }
        }
        public string FilterDialect
        {
            get { return _filterDialect; }
        }
        #endregion

        #region Constructors
        public Subscribe() : this(new Delivery(), null)
        {
        }
        public Subscribe(Delivery delivery, EndpointAddress endTo)
        {
            Delivery = delivery;
            EndTo = endTo;
        }
        public Subscribe(Subscribe subscribe)
        {
            this.Copy(subscribe);
        }
        public Subscribe(XmlReader reader)
        {
            this.ReadXml(reader);
        }
        public void Copy(Subscribe subscribe)
        {
            if (subscribe != null)
            {
                this.EndTo = subscribe.EndTo;
                this.Delivery = subscribe.Delivery;
                this.Expires = subscribe.Expires;
                this.Filter = subscribe.Filter;
            }
        }
        #endregion

        #region IXmlSerializable Members
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            reader.ReadStartElement(WSEventing.ElementNames.Subscribe, WSEventing.NamespaceUri);
            while (reader.NodeType != XmlNodeType.EndElement)
            {
                if (reader.IsStartElement(WSEventing.ElementNames.EndTo, WSEventing.NamespaceUri))
                    EndTo = EndpointAddress.ReadFrom(AddressingVersion.WSAddressing10, reader);
                else if (reader.IsStartElement(WSEventing.ElementNames.Delivery, WSEventing.NamespaceUri))
                    Delivery = new Delivery(reader);
                else if (reader.IsStartElement(WSEventing.ElementNames.Expires, WSEventing.NamespaceUri))
                    Expires = new Expires(reader);
                else if (reader.IsStartElement(WSEventing.ElementNames.Filter, WSEventing.NamespaceUri))
                {
                    string dialect = reader.GetAttribute("Dialect");
                    if (dialect == null || dialect == WSEventing.XPathDialect)
                    {
                        Filter = new XPathMessageFilter(reader);
                    }
                    else
                    {
                        _filterDialect = dialect;
                        reader.Skip();
                    }                      
                }
                reader.MoveToContent();
            }
            reader.ReadEndElement();
        }
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteStartElement(WSEventing.NamespacePrefix, WSEventing.ElementNames.Subscribe, WSEventing.NamespaceUri);
            if (EndTo != null)
                EndTo.WriteTo(AddressingVersion.WSAddressing10, writer, WSEventing.ElementNames.EndTo, WSEventing.NamespaceUri);
            if (Delivery != null)
                Delivery.WriteXml(writer);
            if (Expires != null)
                Expires.WriteXml(writer);
            if (Filter != null)
                Filter.WriteXPathTo(writer, WSEventing.NamespacePrefix, WSEventing.ElementNames.Filter, WSEventing.NamespaceUri, true);
            writer.WriteEndElement();
        }
        #endregion
    }
    #endregion 


    #region SubscribeResult
    [MessageContract]
    [XmlSchemaProvider(null, IsAny = true)]
    public class SubscribeResult : IXmlSerializable
    {
        #region DataMembers
        SubscriptionManager _manager;
        Expires _expires;
        #endregion

        #region Properties
        public SubscriptionManager SubscriptionManager
        {
            get { return _manager; }
            set { _manager = value; }
        }
        public Expires Expires
        {
            get { return _expires; }
            set { _expires = value; }
        }
        #endregion

        #region Constructors
        public SubscribeResult()
        {
        }
        public SubscribeResult(SubscribeResult result)
        {
            this.Copy(result);
        }
        public SubscribeResult(SubscriptionManager manager, Expires expires)
        {
            SubscriptionManager = manager;
            Expires = expires;
        }
        public SubscribeResult(XmlReader reader)
        {
            this.ReadXml(reader);
        }
        public void Copy(SubscribeResult result)
        {
            if (result != null)
            {
                SubscriptionManager = result.SubscriptionManager;
                Expires = result.Expires;
            }
        }
        #endregion

        #region IXmlSerializable Members
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            reader.ReadStartElement(WSEventing.ElementNames.SubscribeResponse, WSEventing.NamespaceUri);
            SubscriptionManager = new SubscriptionManager(reader);
            Expires = new Expires(reader);
            reader.ReadEndElement();
        }
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteStartElement(WSEventing.NamespacePrefix, WSEventing.ElementNames.SubscribeResponse, WSEventing.NamespaceUri);
            if (SubscriptionManager != null)
            {
                SubscriptionManager.WriteXml(writer);
            }
            if (Expires != null)
            {
                Expires.WriteXml(writer);
            }
            writer.WriteEndElement();
        }
        #endregion
    }
    #endregion
}