//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)       
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
#region References
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.Runtime.Serialization;
#endregion


namespace RKiss.WSEventing
{
    [XmlSchemaProvider(null, IsAny = true)]
    public class Subscription : IXmlSerializable
    {
        private EndpointAddress _endTo;
        private Delivery _delivery;
        private DateTime _expires;
        private XPathMessageFilter _filter;
        SubscriptionManager _manager;

        // more properties ...
        private bool _readonly = false;
 
        #region Properties
        public bool IsReadOnly 
        {
            get { return _readonly; }
            set { _readonly = value; }
        }
        public SubscriptionManager Manager
        {
            get { return _manager; }
            set { _manager = value; }
        }
        public EndpointAddress EndTo
        {
            get { return _endTo; }
            set { _endTo = value; }
        }
        public Delivery Delivery
        {
            get { return _delivery; }
            set { _delivery = value; }
        }
        public DateTime Expires
        {
            get { return _expires; }
            set { _expires = value; }
        }
        public XPathMessageFilter Filter
        {
            get { return _filter; }
            set { _filter = value; }
        }
        #endregion

        public Subscription()
        {
        }
        public Subscription(Subscribe subscribe, SubscriptionManager manager)
        {
            Manager = manager;
            EndTo = subscribe.EndTo;
            Delivery = subscribe.Delivery;
            Expires = subscribe.Expires.Value;
            Filter = subscribe.Filter;
        }
        public Subscription(XmlReader reader)
        {
            this.ReadXml(reader);
        }
   
        #region IXmlSerializable Members
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            reader.ReadStartElement(WSEventing.ElementNames.Subscription, WSEventing.NamespaceUri);
            while (reader.NodeType != XmlNodeType.EndElement)
            {
                if (reader.IsStartElement(WSEventing.ElementNames.SubscriptionManager, WSEventing.NamespaceUri))
                    Manager = new SubscriptionManager(reader);
                else if (reader.IsStartElement(WSEventing.ElementNames.EndTo, WSEventing.NamespaceUri))
                    EndTo = EndpointAddress.ReadFrom(AddressingVersion.WSAddressing10, reader);
                else if (reader.IsStartElement(WSEventing.ElementNames.Delivery, WSEventing.NamespaceUri))
                    Delivery = new Delivery(reader);
                else if (reader.IsStartElement(WSEventing.ElementNames.Expires, WSEventing.NamespaceUri))
                    Expires = reader.ReadElementContentAsDateTime();
                else if (reader.IsStartElement(WSEventing.ElementNames.Filter, WSEventing.NamespaceUri))
                    Filter = new XPathMessageFilter(reader);
                reader.MoveToContent();
            }
            reader.ReadEndElement();
        }
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteStartElement(WSEventing.NamespacePrefix, WSEventing.ElementNames.Subscription, WSEventing.NamespaceUri);
            if (Manager != null)
                Manager.WriteXml(writer);
            if (EndTo != null)
                EndTo.WriteTo(AddressingVersion.WSAddressing10, writer, WSEventing.ElementNames.EndTo, WSEventing.NamespaceUri);
            if (Delivery != null)
                Delivery.WriteXml(writer);
            if (Expires != DateTime.MinValue)
                writer.WriteElementString(WSEventing.ElementNames.Expires, WSEventing.NamespaceUri, XmlConvert.ToString(Expires,XmlDateTimeSerializationMode.Utc));
            if (Filter != null)
                Filter.WriteXPathTo(writer, WSEventing.NamespacePrefix, WSEventing.ElementNames.Filter, WSEventing.NamespaceUri, true);
            writer.WriteEndElement();
        }
        #endregion
    }
}

