//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)        
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
#region References
using System;
using System.Globalization;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.ServiceModel;
using System.Runtime.Serialization;
#endregion


namespace RKiss.WSEventing
{
    #region DataContract
    #region SubscriptionEnd
    [XmlSchemaProvider(null, IsAny = true)]
    public class SubscriptionEnd : IXmlSerializable
    {
        #region Properties
        SubscriptionManager _subscriptionManager;
        SubscriptionEndCode _code;
        CultureInfo _cultureInfo;
        string _reason = null;

        public SubscriptionManager SubscriptionManager
        {
            get { return _subscriptionManager; }
            set { _subscriptionManager = value; }
        }
        public SubscriptionEndCode Code
        {
            get { return _code; }
            set { _code = value; }
        }
        public CultureInfo CultureInfo
        {
            get { return _cultureInfo; }
            set { _cultureInfo = value; }
        }
        public string Reason
        {
            get { return _reason; }
            set { _reason = value; }
        }
        #endregion

        #region Constructors
        public SubscriptionEnd()
        {
        }
        public SubscriptionEnd(SubscriptionManager manager) :
           this(manager, SubscriptionEndCode.None, string.Empty, CultureInfo.CurrentCulture)
        {
        }
        public SubscriptionEnd(SubscriptionManager manager, SubscriptionEndCode code, string reason) :
            this(manager, code, reason, CultureInfo.CurrentCulture)
        {
        }
        public SubscriptionEnd(SubscriptionManager manager, SubscriptionEndCode code, string reason, CultureInfo cultureinfo)
        {
            SubscriptionManager = manager;
            Code = code;
            Reason = reason;
            CultureInfo = cultureinfo == null ? CultureInfo.CurrentCulture : cultureinfo;            
        }
        public SubscriptionEnd(XmlReader reader)
        {
            this.ReadXml(reader);
        }
        #endregion

        #region IXmlSerializable Members
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }
        public void ReadXml(XmlReader reader)
        {
            reader.ReadStartElement(WSEventing.ElementNames.SubscriptionEnd, WSEventing.NamespaceUri);
            SubscriptionManager = new SubscriptionManager(reader);
            string code = reader.ReadElementString(WSEventing.ElementNames.Code, WSEventing.NamespaceUri);
            reader.ReadStartElement(WSEventing.ElementNames.Reason, WSEventing.NamespaceUri);
            CultureInfo = new CultureInfo(reader.GetAttribute("lang"));
            Reason = reader.ReadString();
            reader.ReadEndElement();
            reader.ReadEndElement();
        }
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteStartElement(WSEventing.NamespacePrefix, WSEventing.ElementNames.SubscriptionEnd, WSEventing.NamespaceUri);
            if (SubscriptionManager != null)
                SubscriptionManager.WriteXml(writer);
            writer.WriteElementString(WSEventing.NamespacePrefix, WSEventing.ElementNames.Code, WSEventing.NamespaceUri, Code.ToString());
            if (Reason != null)
            {
                writer.WriteStartElement(WSEventing.NamespacePrefix, WSEventing.ElementNames.Reason, WSEventing.NamespaceUri);
                writer.WriteAttributeString("xml", "lang", "http://www.w3.org/XML/1998/namespace", CultureInfo.Name);
                writer.WriteString(Reason);
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
        }
        #endregion
    }
    #endregion

   
    #endregion
}

