//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)         
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
#region References
using System;
using System.Configuration;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.ServiceModel;
using System.Xml;
using System.Runtime.Serialization;
using System.Xml.Serialization;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Channels;
//
using RKiss.WSLib;
using RKiss.WSEventing.Adapters;
#endregion

namespace RKiss.WSEventing
{
    public class SubscriptionManagerClient : ClientBase<IWSEventing>, IWSEventing
    {
        #region Constructors
        public SubscriptionManagerClient(){}
      
        public SubscriptionManagerClient(string configurationName)
            : base(configurationName){}

        public SubscriptionManagerClient(Binding binding, EndpointAddress address)
            : base(binding, address) { }
        #endregion

        #region IWSEventing Members
        public SubscribeResponse Subscribe(SubscribeRequest request)
        {
            return base.Channel.Subscribe(request);
        }

        public GetStatusResponse GetStatus(GetStatusRequest request)
        {
            return base.Channel.GetStatus(request);
        }

        public RenewResponse Renew(RenewRequest request)
        {
            return base.Channel.Renew(request);
        }

        public UnsubscribeResponse Unsubscribe(UnsubscribeRequest request)
        {
            return base.Channel.Unsubscribe(request);
        }
        #endregion

        #region Advanced methods
        public SubscribeResponse Subscribe(string subscriptionTopic, EndpointAddress notifyTo, EndpointAddress endTo, Expires expires, XPathMessageFilter filter)
        {
            SubscribeRequest request = new SubscribeRequest();
            request.SubscriptionTopic = subscriptionTopic;
            request.Delivery.NotifyTo = notifyTo;
            request.EndTo = endTo;
            request.Expires = expires;
            request.Filter = filter;
            return Subscribe(request);
        }
        public SubscribeResponse Subscribe(string subscriptionTopic, EndpointAddress notifyTo, EndpointAddress endTo, DateTime expires)
        {
            return Subscribe(subscriptionTopic, notifyTo, endTo, new Expires(expires), null);
        }
        public SubscribeResponse Subscribe(string subscriptionTopic, EndpointAddress notifyTo, EndpointAddress endTo, TimeSpan duration)
        {
            return Subscribe(subscriptionTopic, notifyTo, endTo, new Expires(duration), null);
        }
        public SubscribeResponse Subscribe(string subscriptionTopic, EndpointAddress notifyTo, DateTime expires)
        {
           return  Subscribe(subscriptionTopic, notifyTo, null, new Expires(expires), null); 
        }
        public SubscribeResponse Subscribe(string subscriptionTopic, EndpointAddress notifyTo, TimeSpan duration)
        {
            return this.Subscribe(subscriptionTopic, notifyTo, null, new Expires(duration), null); 
        }

        public DateTime GetStatus(Identifier subscriptionIdentifier)
        {
            GetStatusRequest request = new GetStatusRequest(subscriptionIdentifier);
            GetStatusResponse response = this.GetStatus(request);
            return response.Expires.Value;
        }

        public DateTime Renew(Identifier subscriptionIdentifier, DateTime expires)
        {
            RenewRequest request = new RenewRequest();
            request.Identifier = subscriptionIdentifier;
            request.Expires = new Expires(expires);
            RenewResponse response = this.Renew(request);
            return response.Expires.Value;
        }

        public DateTime Renew(Identifier subscriptionIdentifier, TimeSpan duration)
        {
            RenewRequest request = new RenewRequest();
            request.Identifier = subscriptionIdentifier;
            request.Expires = new Expires(duration);
            RenewResponse response = this.Renew(request);
            return response.Expires.Value;
        }

        public void Unsubscribe(Identifier subscriptionIdentifier)
        {
            this.Unsubscribe(subscriptionIdentifier);
        }
        #endregion
    }
}