//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)       
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
#region References
using System;
using System.Configuration;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Xml;
using System.Runtime.Serialization;
using System.Xml.Serialization;
//
using RKiss.WSLib;
using RKiss.WSEventing.Adapters;
#endregion

namespace RKiss.WSEventing
{
    /// <summary>
    /// Default WS-Transfer Service
    /// </summary>
    //public class SubscriptionManagerService : SubscriptionManagerService<MemoryStorageAdapter> { }

    public class SubscriptionManagerService : SubscriptionManagerService<ConfiguredAddapter> { }

    [ServiceBehavior(IncludeExceptionDetailInFaults = true)]
    public class SubscriptionManagerService<T> : ServiceAdapterBase<T, IStorageAdapter>, IWSEventing where T: class
    {
        List<string> _delivering = new List<string>();
        List<string> _filtering = new List<string>();

        public SubscriptionManagerService()
        {
            // supported features
            // filtering
            _filtering.Add(WSEventing.XPathDialect);

            // delivery modes (note, that the push mode is a mandatory mode)
            _delivering.Add(WSEventing.PushDeliveryMode);
            MessageProperties mp = OperationContext.Current.IncomingMessageProperties;
            if (mp.ContainsKey("EventingPublishers"))
            {
                Publishers publishers = mp["EventingPublishers"] as Publishers;
                if (publishers != null && publishers.Count > 0)
                {
                    string[] modes = new string[publishers.Count];
                    publishers.Keys.CopyTo(modes, 0);
                    foreach (string mode in modes)
                    {
                        if (mode != WSEventing.PushDeliveryMode)
                            _delivering.Add(mode);
                    }
                }
            }                     
        }

        public SubscribeResponse Subscribe(SubscribeRequest request)
        {
            // validate expiration type and value
            if (request.Expires != null)
            {
                request.Expires.IsValidTime(true);
            }

            // validate delivery mode
            SupportedDeliveryMode sdm = new SupportedDeliveryMode(_delivering);
            sdm.IsSupported(request.Delivery.DeliveryMode, true);

            // validate filtering 
            SupportedDialect sd = new SupportedDialect(_filtering);
            sd.IsSupported(request.FilterDialect, true);

            // response values
            Identifier identifier = new Identifier();
            Expires expires = request.Expires;
            Uri address = OperationContext.Current.IncomingMessageHeaders.To;
            SubscriptionManager manager = new SubscriptionManager(address, identifier);

            // storage resource descriptor
            ResourceDescriptor rd = new ResourceDescriptor(identifier.Value, expires.Value);
            
            // option: subscription topic
            rd.Topic = request.SubscriptionTopic;
           
            // Subscription - storage resource
            Subscription subscription = new Subscription(request, manager);

            //Util.ShowMe(subscription);

            // storage action 
            Adapter.Create<Subscription>(rd, subscription);
           
            // response
            SubscribeResponse response = new SubscribeResponse();
            response.SubscriptionManager = manager;
            response.Expires = expires; 
            return response;
        }
   
        public UnsubscribeResponse Unsubscribe(UnsubscribeRequest request)
        {
            // storage action
            Adapter.Delete<Subscription>(new ResourceDescriptor(request.Identifier.Value));

            UnsubscribeResponse response = new UnsubscribeResponse();
            return response;
        }

        public RenewResponse Renew(RenewRequest request)
        {
            // validate expiration type and value
            if (request.Expires != null)
            {
               request.Expires.IsValidTime(true);
            }
            else
            {
                // never expire
                request.Expires = new Expires(DateTime.MaxValue);
            }

            // storage action
            ResourceDescriptor rd = new ResourceDescriptor(request.Identifier.Value, request.Expires.Value);
            Adapter.Expires(rd);

            // response
            RenewResponse response = new RenewResponse();
            response.Expires = new Expires(rd.Expires);
            return response;
        }

        public GetStatusResponse GetStatus(GetStatusRequest request)
        {
            // storage action
            ResourceDescriptor rd = new ResourceDescriptor(request.Identifier.Value);
            rd = Adapter.GetResourceDescriptor(rd);

            // response
            GetStatusResponse response = new GetStatusResponse();
            response.Expires = new Expires(rd.Expires);
            return response;
        }   
    }
}



