//*****************************************************************************
//    Description.....WS-Eventing for WCF
//                                
//    Author..........Roman Kiss, rkiss@pathcom.com
//    Copyright  2005 ATZ Consulting Inc. (see included license.rtf file)       
//                        
//    Date Created:    06/06/06
//
//    Date        Modified By     Description
//-----------------------------------------------------------------------------
//    06/06/06    Roman Kiss     Initial Revision
//    07/07/06    Roman Kiss     Version JuneCTP 2006
//    11/11/06    Roman Kiss     RTM
//*****************************************************************************
//  
using System.Xml;
using System.ServiceModel;

namespace RKiss.WSEventing
{
    public sealed class WSEventing
    {
        public const string NamespacePrefix = "wse";
        public const string NamespaceUri = "http://schemas.xmlsoap.org/ws/2004/08/eventing";
        
        public const string SubscribeAction = NamespaceUri + "/" + ElementNames.Subscribe;
        public const string SubscribeResponseAction = NamespaceUri + "/" + ElementNames.SubscribeResponse;
        public const string UnsubscribeAction = NamespaceUri + "/" + ElementNames.Unsubscribe;
        public const string UnsubscribeResponseAction = NamespaceUri + "/" + ElementNames.UnsubscribeResponse;
        public const string RenewAction = NamespaceUri + "/" + ElementNames.Renew;
        public const string RenewResponseAction = NamespaceUri + "/" + ElementNames.RenewResponse;
        public const string GetStatusAction = NamespaceUri + "/" + ElementNames.GetStatus;
        public const string GetStatusResponseAction = NamespaceUri + "/" + ElementNames.GetStatusResponse;
        public const string SubscriptionEndAction = NamespaceUri + "/" + ElementNames.SubscriptionEnd;     
        public const string PushDeliveryMode = NamespaceUri + "/DeliveryModes/Push";
        public const string WrappedDeliveryMode = NamespaceUri + "/DeliveryModes/Wrap";
        public const string XPathDialect = "http://www.w3.org/TR/1999/REC-xpath-19991116";
        public const string DeliveryFailure = NamespaceUri + "/DeliveryFailure";
        public const string SourceShuttingDown = NamespaceUri + "/SourceShuttingDown";
        public const string SourceCanceling = NamespaceUri + "/SourceCanceling";

        
        public sealed class ElementNames
        {
            public const string Code = "Code";
            public const string Delivery = "Delivery";
            public const string Dialect = "Dialect";
            public const string EndTo = "EndTo";
            public const string Eventing = "Eventing";
            public const string EventSource = "EventSource";
            public const string Expires = "Expires";
            public const string Filter = "Filter";
            public const string GetStatus = "GetStatus";
            public const string GetStatusResponse = "GetStatusResponse";
            public const string Identifier = "Identifier";
            public const string Mode = "Mode";
            public const string NotifyTo = "NotifyTo";
            public const string Reason = "Reason";
            public const string Renew = "Renew";
            public const string RenewResponse = "RenewResponse";
            public const string Status = "Status";
            public const string Subscribe = "Subscribe";
            public const string SubscriptionEnd = "SubscriptionEnd";
            public const string SubscriptionManager = "SubscriptionManager";
            public const string SubscribeResponse = "SubscribeResponse";
            public const string Unsubscribe = "Unsubscribe";
            public const string UnsubscribeResponse = "UnsubscribeResponse";
            public const string SupportedDeliveryMode = "SupportedDeliveryMode";
            public const string SupportedDialect = "SupportedDialect";
            
            // misc - extenstion
            public const string Subscription = "Subscription";
            public const string EventTopic = "EventTopic";
        }

        // Subscribe fault
        public sealed class DeliveryModeRequestedUnvailable
        {
           public const string SubCode = "DeliveryModeRequestedUnvailable";
           public const string Reason = "The requested delivery mode is not supported.";     
        }

        // Subscribe fault 
        public sealed class InvalidExpirationType
        {
            public const string SubCode = "InvalidExpirationType";
            public const string Reason = "The expiration time requested is invalid.";
        }

        // Subscribe fault
        public sealed class UnsupportedExpirationType
        {
            public const string SubCode = "UnsupportedExpirationType";
            public const string Reason = "Only expiration durations are supported.";
        }

        // Subscribe fault
        public sealed class FilteringNotSupported
        {
            public const string SubCode = "FilteringNotSupported";
            public const string Reason = "Filtering is not supported.";
        }

        // Subscribe fault
        public sealed class FilteringRequestedUnavailable
        {
            public const string SubCode = "FilteringRequestedUnavailable";
            public const string Reason = "The requested filter dialect is not supported.";
        }

        // Subscribe fault
        public sealed class EventSourceUnableToProcess
        {
            public const string SubCode = "EventSourceUnableToProcess";
        }

        // Renew fault
        public sealed class UnableToRenew
        {
            public const string SubCode = "UnableToRenew";
        }

        // Any
        public sealed class InvalidMessage
        {
            public const string SubCode = "InvalidMessage";
            public const string Reason = "The message is not valid and cannot be processed.";
        }

        public sealed class Extension
        {
            // extension
            public const string SubscriptionTopic = "SubscriptionTopic";
            public const string NamespaceUri = "http://www.rkiss.net/schemas/sb/2004/08/servicebus";
            public const string NamespacePrefix = "rksb";
        }
    }

    public enum SubscriptionEndCode
    {
        DeliveryFailure,
        SourceShuttingDown,
        SourceCanceling,
        None
    }


}