using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using RKiss.WSEventing;

namespace ReceivingServer
{
    class EventingProxy
    {
        private static Subscriber instance;
        private SubscriptionManagerClient manager;
        private Identifier id;


        public EventingProxy()
        {
            manager = new SubscriptionManagerClient("SubscriptionManager");         

        }

        public SubscribeResponse Subscribe()
        {
            // Prepare request for subscribing subscription
            SubscribeRequest subscribeRequest = new SubscribeRequest();

            // option: Application specific header
            //AddressHeader[] ah = new AddressHeader[1] { AddressHeader.CreateAddressHeader("MySubscription", "urn:MyNamespace", "1234567890") };

            // option: organize subscriptions in the storage based on the topic
            subscribeRequest.SubscriptionTopic = "messageTest";
            // or another option
            //subscribeRequest.Delivery.Add(AddressHeader.CreateAddressHeader("EventTopic", "", "weather:report.storm"));

            // Delivery:
            subscribeRequest.Delivery.DeliveryMode = WSEventing.PushDeliveryMode;
            EndpointAddress notifyTo = new EndpointAddress(new Uri("http://localhost:33333/OnNewMessage"));
            subscribeRequest.Delivery.NotifyTo = notifyTo;

            // EndTo:
            //subscribeRequest.EndTo = new EndpointAddress(new Uri("http://localhost:44444/Admin"));

            // Filter:
            //XmlNamespaceManager nsmgr = new XmlNamespaceManager(new XmlDocument().NameTable);
            //nsmgr.AddNamespace("ow", "http://www.example.org/oceanwatch");
            //subscribeRequest.Filter = new XPathMessageFilter("//ow:WindReport/ow:Speed >= 65", nsmgr);

            // lifetime of the subscription
            subscribeRequest.Expires = new Expires(TimeSpan.FromSeconds(36000));
            //subscribeRequest.Expires = new Expires(new DateTime(2007, 05, 15));
            //subscribeRequest.Subscribe.Expires = new Expires(DateTime.Now.Add(TimeSpan.FromSeconds(300)));

            //Util.ShowMe(subscribeRequest);

            // action - Subscribe
            SubscribeResponse subscribeResponse = manager.Subscribe(subscribeRequest);

            // show me response
            //Util.ShowMe(subscribeResponse);

            id = subscribeResponse.SubscriptionManager.Identifier;

            return subscribeResponse;
        }

        public UnsubscribeResponse Unsubscribe()
        {            
            return manager.Unsubscribe(new UnsubscribeRequest(id));
        }

        public RenewResponse Renew()
        {
            RenewRequest renewRequest = new RenewRequest(id, new Renew(new Expires(TimeSpan.FromSeconds(120))));
            return manager.Renew(renewRequest);

        }

        public GetStatusResponse GetStatus()
        {            
            return manager.GetStatus(new GetStatusRequest(id));
        }
    }
}
